/*
*  Name:      defs-uesqlc.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQLC definitions
*  Date:      $Date: 1999/07/26 15:47:38 $
*  Revision:  $Revision: 1.6 $
*
*  Copyright (C) 1996-1999  Rafael Jesus Alcantara Perez <ralcan@vnet.es>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include "defs-uesqlc.hh"


namespace uesql
{

  //
  //  E X P O R T E D   V A R I A B L E S   F O R   U E S Q L C - C O M P I L E R
  //
  //   G E N E R A T E D   S O U R C E S   A N D   C L I E N T   P R O G R A M S
  // 

  ///  Map with \a SQLSTATE value to unique integers correspondences.
  map<string, int>   TSqlstateHandler::_tValueToIntMap;

  ///  SQL state alphanumeric code.
  string             SQLSTATE = "00000";

  ///  SQL numeric code.
  long int           SQLCODE  = 0;


  //
  //  SQLSTATE class and subclass values.
  //    Classes has been represented as a C++ namespace, and subclasses have
  //    been represented as a constant pointer to a zero terminated string.
  //

  namespace ambiguous_cursor_name
  {
    const char*   pkcNO_SUBCLASS = "3C000";
  }

  namespace cardinality_violation
  {
    const char*   pkcNO_SUBCLASS = "21000";
  }
  
  namespace connection_exception
  {
    const char*   pkcNO_SUBCLASS                                       = "08000";
    const char*   pkcCONNECTION_DOES_NOT_EXIST                         = "08003";
    const char*   pkcCONNECTION_FAILURE                                = "08006";
    const char*   pkcCONNECTION_NAME_IN_USE                            = "08002";
    const char*   pkcSQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION       = "08001";
    const char*   pkcSQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION = "08004";
    const char*   pkcTRANSACTION_RESOLUTION_UNKNOWN                    = "08007";
  }
  
  namespace data_exception
  {
    const char*   pkcNO_SUBCLASS                          = "22000";
    const char*   pkcCHARACTER_NOT_IN_REPERTOIRE          = "22021";
    const char*   pkcDATETIME_FIELD_OVERFLOW              = "22008";
    const char*   pkcDIVISION_BY_ZERO                     = "22012";
    const char*   pkcERROR_IN_ASSIGNMENT                  = "22005";
    const char*   pkcINDICATOR_OVERFLOW                   = "22022";
    const char*   pkcINTERVAL_FIELD_OVERFLOW              = "22015";
    const char*   pkcINVALID_CHARACTER_VALUE_FOR_CAST     = "22018";
    const char*   pkcINVALID_DATETIME_FORMAT              = "22007";
    const char*   pkcINVALID_ESCAPE_CHARACTER             = "22019";
    const char*   pkcINVALID_ESCAPE_SEQUENCE              = "22025";
    const char*   pkcINVALID_PARAMETER_VALUE              = "22023";
    const char*   pkcINVALID_TIME_ZONE_DISPLACEMENT_VALUE = "22009";
    const char*   pkcNULL_VALUE_NO_INDICATOR_PARAMETER    = "22002";
    const char*   pkcNUMERIC_VALUE_OUT_OF_RANGE           = "22003";
    const char*   pkcSTRING_DATA_LENGTH_MISMATCH          = "22026";
    const char*   pkcSTRING_DATA_RIGHT_TRUNCATION         = "22001";
    const char*   pkcSUBSTRING_ERROR                      = "22011";
    const char*   pkcTRIM_ERROR                           = "22027";
    const char*   pkcUNTERMINATED_C_STRING                = "22024";
  }
  
  namespace dependent_privilege_descriptors_still_exist
  {
    const char*   pkcNO_SUBCLASS = "2B000";
  }
  
  namespace dynamic_sql_error
  {
    const char*   pkcNO_SUBCLASS                                        = "07000";
    const char*   pkcCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED            = "07003";
    const char*   pkcINVALID_DESCRIPTOR_COUNT                           = "07008";
    const char*   pkcINVALID_DESCRIPTOR_INDEX                           = "07009";
    const char*   pkcPREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION      = "07005";
    const char*   pkcRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION           = "07006";
    const char*   pkcUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC = "07001";
    const char*   pkcUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC            = "07002";
    const char*   pkcUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS       = "07004";
    const char*   pkcUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS            = "07007";
  }
  
  namespace feature_not_supported
  {
    const char*   pkcNO_SUBCLASS                       = "0A000";
    const char*   pkcMULTIPLE_ENVIRONMENT_TRANSACTIONS = "0A001";
  }
  
  namespace integrity_constraint_violation
  {
    const char*   pkcNO_SUBCLASS = "23000";
  }
  
  namespace invalid_authorization_specification
  {
    const char*   pkcNO_SUBCLASS = "28000";
  }
  
  namespace invalid_catalog_name
  {
    const char*   pkcNO_SUBCLASS = "3D000";
  }
  
  namespace invalid_character_set_name
  {
    const char*   pkcNO_SUBCLASS = "2C000";
  }
  
  namespace invalid_condition_number
  {
    const char*   pkcNO_SUBCLASS = "35000";
  }
  
  namespace invalid_connection_name
  {
    const char*   pkcNO_SUBCLASS = "2E000";
  }
  
  namespace invalid_cursor_name
  {
    const char*   pkcNO_SUBCLASS = "34000";
  }
  
  namespace invalid_cursor_state
  {
    const char*   pkcNO_SUBCLASS = "24000";
  }
  
  namespace invalid_schema_name
  {
    const char*   pkcNO_SUBCLASS = "3F000";
  }
  
  namespace invalid_sql_descriptor_name
  {
    const char*   pkcNO_SUBCLASS = "33000";
  }
  
  namespace invalid_sql_statement_name
  {
    const char*   pkcNO_SUBCLASS = "26000";
  }
  
  namespace invalid_transaction_state
  {
    const char*   pkcNO_SUBCLASS = "25000";
  }
  
  namespace invalid_transaction_termination
  {
    const char*   pkcNO_SUBCLASS = "2D000";
  }
  
  namespace no_data
  {
    const char*   pkcNO_SUBCLASS = "02000";
  }
  
  namespace remote_database_access
  {
    const char*   pkcNO_SUBCLASS = "HZ000";
  }
  
  namespace successful_completion
  {
    const char*   pkcNO_SUBCLASS = "00000";
  }
  
  namespace syntax_error_or_access_rule_violation
  {
    const char*   pkcNO_SUBCLASS = "42000";
  }
  
  namespace syntax_error_or_access_rule_violation_in_direct_statement
  {
    const char*   pkcNO_SUBCLASS = "2A000";
  }
  
  namespace syntax_error_or_access_rule_violation_in_dynamic_statement
  {
    const char*   pkcNO_SUBCLASS = "37000";
  }
  
  namespace transaction_rollback
  {
    const char*   pkcNO_SUBCLASS                    = "40000";
    const char*   pkcINTEGRITY_CONSTRAINT_VIOLATION = "40002";
    const char*   pkcSERIALIZATION_FAILURE          = "40001";
    const char*   pkcSTATEMENT_COMPLETION_UNKNOWN   = "40003";
  }
  
  namespace triggered_data_change_violation
  {
    const char*   pkcNO_SUBCLASS = "27000";
  }
  
  namespace warning
  {
    const char*   pkcNO_SUBCLASS                                      = "01000";
    const char*   pkcCURSOR_OPERATION_CONFLICT                        = "01001";
    const char*   pkcDISCONNECT_ERROR                                 = "01002";
    const char*   pkcIMPLICIT_ZERO_BIT_PADDING                        = "01008";
    const char*   pkcINSUFFICIENT_ITEM_DESCRIPTOR_AREAS               = "01005";
    const char*   pkcNULL_VALUE_ELIMINATED_IN_SET_FUNCTION            = "01003";
    const char*   pkcPRIVILEGE_NOT_GRANTED                            = "01007";
    const char*   pkcPRIVILEGE_NOT_REVOKED                            = "01006";
    const char*   pkcQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA = "0100A";
    const char*   pkcSEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA = "01009";
    const char*   pkcSTRING_DATA_RIGHT_TRUNCATION                     = "01004";
  }
  
  namespace with_check_option_violation
  {
    const char*   pkcNO_SUBCLASS = "44000";
  }

}  // namespace uesql


//
//  L O C A L   F U N C T I O N S
//

void uesql::TSqlstateHandler::_initialize (void)
{

# define INSERT_PAIR_FOR(NAMESPACE, VALUE)                                  \
         _tValueToIntMap.insert (make_pair ( NAMESPACE ## ::pkc ## VALUE ,  \
                                             NAMESPACE ## ::e ## VALUE   ))

  if ( _tValueToIntMap.empty() )
  {
    INSERT_PAIR_FOR (ambiguous_cursor_name, NO_SUBCLASS);
    INSERT_PAIR_FOR (cardinality_violation, NO_SUBCLASS);
    INSERT_PAIR_FOR (connection_exception, NO_SUBCLASS);
    INSERT_PAIR_FOR (connection_exception, CONNECTION_DOES_NOT_EXIST);
    INSERT_PAIR_FOR (connection_exception, CONNECTION_FAILURE);
    INSERT_PAIR_FOR (connection_exception, CONNECTION_NAME_IN_USE);
    INSERT_PAIR_FOR (connection_exception, SQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION);
    INSERT_PAIR_FOR (connection_exception, SQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION);
    INSERT_PAIR_FOR (connection_exception, TRANSACTION_RESOLUTION_UNKNOWN);
    INSERT_PAIR_FOR (data_exception, NO_SUBCLASS);
    INSERT_PAIR_FOR (data_exception, CHARACTER_NOT_IN_REPERTOIRE);
    INSERT_PAIR_FOR (data_exception, DATETIME_FIELD_OVERFLOW);
    INSERT_PAIR_FOR (data_exception, DIVISION_BY_ZERO);
    INSERT_PAIR_FOR (data_exception, ERROR_IN_ASSIGNMENT);
    INSERT_PAIR_FOR (data_exception, INDICATOR_OVERFLOW);
    INSERT_PAIR_FOR (data_exception, INTERVAL_FIELD_OVERFLOW);
    INSERT_PAIR_FOR (data_exception, INVALID_CHARACTER_VALUE_FOR_CAST);
    INSERT_PAIR_FOR (data_exception, INVALID_DATETIME_FORMAT);
    INSERT_PAIR_FOR (data_exception, INVALID_ESCAPE_CHARACTER);
    INSERT_PAIR_FOR (data_exception, INVALID_ESCAPE_SEQUENCE);
    INSERT_PAIR_FOR (data_exception, INVALID_PARAMETER_VALUE);
    INSERT_PAIR_FOR (data_exception, INVALID_TIME_ZONE_DISPLACEMENT_VALUE);
    INSERT_PAIR_FOR (data_exception, NULL_VALUE_NO_INDICATOR_PARAMETER);
    INSERT_PAIR_FOR (data_exception, NUMERIC_VALUE_OUT_OF_RANGE);
    INSERT_PAIR_FOR (data_exception, STRING_DATA_LENGTH_MISMATCH);
    INSERT_PAIR_FOR (data_exception, STRING_DATA_RIGHT_TRUNCATION);
    INSERT_PAIR_FOR (data_exception, SUBSTRING_ERROR);
    INSERT_PAIR_FOR (data_exception, TRIM_ERROR);
    INSERT_PAIR_FOR (data_exception, UNTERMINATED_C_STRING);
    INSERT_PAIR_FOR (dependent_privilege_descriptors_still_exist, NO_SUBCLASS);
    INSERT_PAIR_FOR (dynamic_sql_error, NO_SUBCLASS);
    INSERT_PAIR_FOR (dynamic_sql_error, CURSOR_SPECIFICATION_CANNOT_BE_EXECUTED);
    INSERT_PAIR_FOR (dynamic_sql_error, INVALID_DESCRIPTOR_COUNT);
    INSERT_PAIR_FOR (dynamic_sql_error, INVALID_DESCRIPTOR_INDEX);
    INSERT_PAIR_FOR (dynamic_sql_error, PREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION);
    INSERT_PAIR_FOR (dynamic_sql_error, RESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION);
    INSERT_PAIR_FOR (dynamic_sql_error, USING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC);
    INSERT_PAIR_FOR (dynamic_sql_error, USING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC);
    INSERT_PAIR_FOR (dynamic_sql_error, USING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS);
    INSERT_PAIR_FOR (dynamic_sql_error, USING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS);
    INSERT_PAIR_FOR (feature_not_supported, NO_SUBCLASS);
    INSERT_PAIR_FOR (feature_not_supported, MULTIPLE_ENVIRONMENT_TRANSACTIONS);
    INSERT_PAIR_FOR (integrity_constraint_violation, NO_SUBCLASS);
    INSERT_PAIR_FOR (invalid_authorization_specification, NO_SUBCLASS);
    INSERT_PAIR_FOR (invalid_catalog_name, NO_SUBCLASS);
    INSERT_PAIR_FOR (invalid_character_set_name, NO_SUBCLASS);
    INSERT_PAIR_FOR (invalid_condition_number, NO_SUBCLASS);
    INSERT_PAIR_FOR (invalid_connection_name, NO_SUBCLASS);
    INSERT_PAIR_FOR (invalid_cursor_name, NO_SUBCLASS);
    INSERT_PAIR_FOR (invalid_cursor_state, NO_SUBCLASS);
    INSERT_PAIR_FOR (invalid_schema_name, NO_SUBCLASS);
    INSERT_PAIR_FOR (invalid_sql_descriptor_name, NO_SUBCLASS);
    INSERT_PAIR_FOR (invalid_sql_statement_name, NO_SUBCLASS);
    INSERT_PAIR_FOR (invalid_transaction_state, NO_SUBCLASS);
    INSERT_PAIR_FOR (invalid_transaction_termination, NO_SUBCLASS);
    INSERT_PAIR_FOR (no_data, NO_SUBCLASS);
    INSERT_PAIR_FOR (remote_database_access, NO_SUBCLASS);
    INSERT_PAIR_FOR (successful_completion, NO_SUBCLASS);
    INSERT_PAIR_FOR (syntax_error_or_access_rule_violation, NO_SUBCLASS);
    INSERT_PAIR_FOR (syntax_error_or_access_rule_violation_in_direct_statement, NO_SUBCLASS);
    INSERT_PAIR_FOR (syntax_error_or_access_rule_violation_in_dynamic_statement, NO_SUBCLASS);
    INSERT_PAIR_FOR (transaction_rollback, NO_SUBCLASS);
    INSERT_PAIR_FOR (transaction_rollback, INTEGRITY_CONSTRAINT_VIOLATION);
    INSERT_PAIR_FOR (transaction_rollback, SERIALIZATION_FAILURE);
    INSERT_PAIR_FOR (transaction_rollback, STATEMENT_COMPLETION_UNKNOWN);
    INSERT_PAIR_FOR (triggered_data_change_violation, NO_SUBCLASS);
    INSERT_PAIR_FOR (warning, NO_SUBCLASS);
    INSERT_PAIR_FOR (warning, CURSOR_OPERATION_CONFLICT);
    INSERT_PAIR_FOR (warning, DISCONNECT_ERROR);
    INSERT_PAIR_FOR (warning, IMPLICIT_ZERO_BIT_PADDING);
    INSERT_PAIR_FOR (warning, INSUFFICIENT_ITEM_DESCRIPTOR_AREAS);
    INSERT_PAIR_FOR (warning, NULL_VALUE_ELIMINATED_IN_SET_FUNCTION);
    INSERT_PAIR_FOR (warning, PRIVILEGE_NOT_GRANTED);
    INSERT_PAIR_FOR (warning, PRIVILEGE_NOT_REVOKED);
    INSERT_PAIR_FOR (warning, QUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA);
    INSERT_PAIR_FOR (warning, SEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA);
    INSERT_PAIR_FOR (warning, STRING_DATA_RIGHT_TRUNCATION);
    INSERT_PAIR_FOR (with_check_option_violation, NO_SUBCLASS);
  }

# undef INSERT_PAIR_FOR

}  // _initialize()
  

//
//  C O N S T R U C T O R S
//

const char* uesql::TSqlstateHandler::getClass (const char* pkcSOURCE_SQLSTATE) const
{

  const char*   pkcClass = NULL;
    
  switch (getUniqueValue (pkcSOURCE_SQLSTATE))
  {
    case ambiguous_cursor_name::eNO_SUBCLASS:
    {
      pkcClass = "ambiguous cursor name";
      break;
    }
    case cardinality_violation::eNO_SUBCLASS:
    {
      pkcClass = "cardinality violation";
      break;
    }
    case connection_exception::eNO_SUBCLASS:
    case connection_exception::eCONNECTION_DOES_NOT_EXIST:
    case connection_exception::eCONNECTION_FAILURE:
    case connection_exception::eCONNECTION_NAME_IN_USE:
    case connection_exception::eSQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION:
    case connection_exception::eSQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION:
    case connection_exception::eTRANSACTION_RESOLUTION_UNKNOWN:
    {
      pkcClass = "connection exception";
      break;
    }
    case data_exception::eNO_SUBCLASS:
    case data_exception::eCHARACTER_NOT_IN_REPERTOIRE:
    case data_exception::eDATETIME_FIELD_OVERFLOW:
    case data_exception::eDIVISION_BY_ZERO:
    case data_exception::eERROR_IN_ASSIGNMENT:
    case data_exception::eINDICATOR_OVERFLOW:
    case data_exception::eINTERVAL_FIELD_OVERFLOW:
    case data_exception::eINVALID_CHARACTER_VALUE_FOR_CAST:
    case data_exception::eINVALID_DATETIME_FORMAT:
    case data_exception::eINVALID_ESCAPE_CHARACTER:
    case data_exception::eINVALID_ESCAPE_SEQUENCE:
    case data_exception::eINVALID_PARAMETER_VALUE:
    case data_exception::eINVALID_TIME_ZONE_DISPLACEMENT_VALUE:
    case data_exception::eNULL_VALUE_NO_INDICATOR_PARAMETER:
    case data_exception::eNUMERIC_VALUE_OUT_OF_RANGE:
    case data_exception::eSTRING_DATA_LENGTH_MISMATCH:
    case data_exception::eSTRING_DATA_RIGHT_TRUNCATION:
    case data_exception::eSUBSTRING_ERROR:
    case data_exception::eTRIM_ERROR:
    case data_exception::eUNTERMINATED_C_STRING:
    {
      pkcClass = "data exception";
      break;
    }
    case dependent_privilege_descriptors_still_exist::eNO_SUBCLASS:
    {
      pkcClass = "dependent privilege descriptors still exist";
      break;
    }
    case dynamic_sql_error::eNO_SUBCLASS:
    case dynamic_sql_error::eCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED:
    case dynamic_sql_error::eINVALID_DESCRIPTOR_COUNT:
    case dynamic_sql_error::eINVALID_DESCRIPTOR_INDEX:
    case dynamic_sql_error::ePREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION:
    case dynamic_sql_error::eRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION:
    case dynamic_sql_error::eUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC:
    case dynamic_sql_error::eUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC:
    case dynamic_sql_error::eUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS:
    case dynamic_sql_error::eUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS:
    {
      pkcClass = "dynamic SQL error";
      break;
    }
    case feature_not_supported::eNO_SUBCLASS:
    case feature_not_supported::eMULTIPLE_ENVIRONMENT_TRANSACTIONS:
    {
      pkcClass = "feature not supported";
      break;
    }
    case integrity_constraint_violation::eNO_SUBCLASS:
    {
      pkcClass = "integrity constraint violation";
      break;
    }
    case invalid_authorization_specification::eNO_SUBCLASS:
    {
      pkcClass = "invalid authorization specification";
      break;
    }
    case invalid_catalog_name::eNO_SUBCLASS:
    {
      pkcClass = "invalid catalog name";
      break;
    }
    case invalid_character_set_name::eNO_SUBCLASS:
    {
      pkcClass = "invalid character set name";
      break;
    }
    case invalid_condition_number::eNO_SUBCLASS:
    {
      pkcClass = "invalid condition number";
      break;
    }
    case invalid_connection_name::eNO_SUBCLASS:
    {
      pkcClass = "invalid connection name";
      break;
    }
    case invalid_cursor_name::eNO_SUBCLASS:
    {
      pkcClass = "invalid cursor name";
      break;
    }
    case invalid_cursor_state::eNO_SUBCLASS:
    {
      pkcClass = "invalid cursor state";
      break;
    }
    case invalid_schema_name::eNO_SUBCLASS:
    {
      pkcClass = "invalid schema name";
      break;
    }
    case invalid_sql_descriptor_name::eNO_SUBCLASS:
    {
      pkcClass = "invalid SQL descriptor name";
      break;
    }
    case invalid_sql_statement_name::eNO_SUBCLASS:
    {
      pkcClass = "invalid SQL statement name";
      break;
    }
    case invalid_transaction_state::eNO_SUBCLASS:
    {
      pkcClass = "invalid transaction state";
      break;
    }
    case invalid_transaction_termination::eNO_SUBCLASS:
    {
      pkcClass = "invalid transaction termination";
      break;
    }
    case no_data::eNO_SUBCLASS:
    {
      pkcClass = "no data";
      break;
    }
    case remote_database_access::eNO_SUBCLASS:
    {
      pkcClass = "remote database access";
      break;
    }
    case successful_completion::eNO_SUBCLASS:
    {
      pkcClass = "successful completion";
      break;
    }
    case syntax_error_or_access_rule_violation::eNO_SUBCLASS:
    {
      pkcClass = "syntax error or access rule violation";
      break;
    }
    case syntax_error_or_access_rule_violation_in_direct_statement::eNO_SUBCLASS:
    {
      pkcClass = "syntax error or access rule violation in direct statement";
      break;
    }
    case syntax_error_or_access_rule_violation_in_dynamic_statement::eNO_SUBCLASS:
    {
      pkcClass = "syntax error or access rule violation in dynamic statement";
      break;
    }
    case transaction_rollback::eNO_SUBCLASS:
    case transaction_rollback::eINTEGRITY_CONSTRAINT_VIOLATION:
    case transaction_rollback::eSERIALIZATION_FAILURE:
    case transaction_rollback::eSTATEMENT_COMPLETION_UNKNOWN:
    {
      pkcClass = "transaction rollback";
      break;
    }
    case triggered_data_change_violation::eNO_SUBCLASS:
    {
      pkcClass = "triggered data change violation";
      break;
    }
    case warning::eNO_SUBCLASS:
    case warning::eCURSOR_OPERATION_CONFLICT:
    case warning::eDISCONNECT_ERROR:
    case warning::eIMPLICIT_ZERO_BIT_PADDING:
    case warning::eINSUFFICIENT_ITEM_DESCRIPTOR_AREAS:
    case warning::eNULL_VALUE_ELIMINATED_IN_SET_FUNCTION:
    case warning::ePRIVILEGE_NOT_GRANTED:
    case warning::ePRIVILEGE_NOT_REVOKED:
    case warning::eQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA:
    case warning::eSEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA:
    case warning::eSTRING_DATA_RIGHT_TRUNCATION:
    {
      pkcClass = "warning";
      break;
    }
    case with_check_option_violation::eNO_SUBCLASS:
    {
      pkcClass = "with check option violation";
      break;
    }
  }
  return pkcClass;
  
}  // getClass()


const char* uesql::TSqlstateHandler::getSubclass (const char* pkcSOURCE_SQLSTATE) const
{
  
  const char*   pkcSubclass = NULL;
  
  switch (getUniqueValue (pkcSOURCE_SQLSTATE))
  {
    case ambiguous_cursor_name::eNO_SUBCLASS:
    case cardinality_violation::eNO_SUBCLASS:
    case connection_exception::eNO_SUBCLASS:
    case data_exception::eNO_SUBCLASS:
    case dependent_privilege_descriptors_still_exist::eNO_SUBCLASS:
    case dynamic_sql_error::eNO_SUBCLASS:
    case feature_not_supported::eNO_SUBCLASS:
    case integrity_constraint_violation::eNO_SUBCLASS:
    case invalid_authorization_specification::eNO_SUBCLASS:
    case invalid_catalog_name::eNO_SUBCLASS:
    case invalid_character_set_name::eNO_SUBCLASS:
    case invalid_condition_number::eNO_SUBCLASS:
    case invalid_connection_name::eNO_SUBCLASS:
    case invalid_cursor_name::eNO_SUBCLASS:
    case invalid_cursor_state::eNO_SUBCLASS:
    case invalid_schema_name::eNO_SUBCLASS:
    case invalid_sql_descriptor_name::eNO_SUBCLASS:
    case invalid_sql_statement_name::eNO_SUBCLASS:
    case invalid_transaction_state::eNO_SUBCLASS:
    case invalid_transaction_termination::eNO_SUBCLASS:
    case no_data::eNO_SUBCLASS:
    case remote_database_access::eNO_SUBCLASS:
    case successful_completion::eNO_SUBCLASS:
    case syntax_error_or_access_rule_violation::eNO_SUBCLASS:
    case syntax_error_or_access_rule_violation_in_direct_statement::eNO_SUBCLASS:
    case syntax_error_or_access_rule_violation_in_dynamic_statement::eNO_SUBCLASS:
    case transaction_rollback::eNO_SUBCLASS:
    case triggered_data_change_violation::eNO_SUBCLASS:
    case warning::eNO_SUBCLASS:
    case with_check_option_violation::eNO_SUBCLASS:
    {
      pkcSubclass = "(no subclass)";
      break;
    }
    case connection_exception::eCONNECTION_DOES_NOT_EXIST:
    {
      pkcSubclass = "connection does not exist";
      break;
    }
    case connection_exception::eCONNECTION_FAILURE:
    {
      pkcSubclass = "connection failure";
      break;
    }
    case connection_exception::eCONNECTION_NAME_IN_USE:
    {
      pkcSubclass = "connection name in use";
      break;
    }
    case connection_exception::eSQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION:
    {
      pkcSubclass = "SQL-client unable to establish SQL-connection";
      break;
    }
    case connection_exception::eSQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION:
    {
      pkcSubclass = "SQL-server rejected establishment of SQL-connection";
      break;
    }
    case connection_exception::eTRANSACTION_RESOLUTION_UNKNOWN:
    {
      pkcSubclass = "transaction resolution unknown";
      break;
    }
    case data_exception::eCHARACTER_NOT_IN_REPERTOIRE:
    {
      pkcSubclass = "character not in repertoire";
      break;
    }
    case data_exception::eDATETIME_FIELD_OVERFLOW:
    {
      pkcSubclass = "datetime field overflow";
      break;
    }
    case data_exception::eDIVISION_BY_ZERO:
    {
      pkcSubclass = "division by zero";
      break;
    }
    case data_exception::eERROR_IN_ASSIGNMENT:
    {
      pkcSubclass = "error in assignment";
      break;
    }
    case data_exception::eINDICATOR_OVERFLOW:
    {
      pkcSubclass = "indicator overflow";
      break;
    }
    case data_exception::eINTERVAL_FIELD_OVERFLOW:
    {
      pkcSubclass = "interval field overflow";
      break;
    }
    case data_exception::eINVALID_CHARACTER_VALUE_FOR_CAST:
    {
      pkcSubclass = "invalid character value for cast";
      break;
    }
    case data_exception::eINVALID_DATETIME_FORMAT:
    {
      pkcSubclass = "invalid datetime format";
      break;
    }
    case data_exception::eINVALID_ESCAPE_CHARACTER:
    {
      pkcSubclass = "invalid escape character";
      break;
    }
    case data_exception::eINVALID_ESCAPE_SEQUENCE:
    {
      pkcSubclass = "invalid escape sequence";
      break;
    }
    case data_exception::eINVALID_PARAMETER_VALUE:
    {
      pkcSubclass = "invalid parameter value";
      break;
    }
    case data_exception::eINVALID_TIME_ZONE_DISPLACEMENT_VALUE:
    {
      pkcSubclass = "invalid time zone displacement value";
      break;
    }
    case data_exception::eNULL_VALUE_NO_INDICATOR_PARAMETER:
    {
      pkcSubclass = "null value, no indicator parameter";
      break;
    }
    case data_exception::eNUMERIC_VALUE_OUT_OF_RANGE:
    {
      pkcSubclass = "numeric value out of range";
      break;
    }
    case data_exception::eSTRING_DATA_LENGTH_MISMATCH:
    {
      pkcSubclass = "string data, length mismatch";
      break;
    }
    case data_exception::eSTRING_DATA_RIGHT_TRUNCATION:
    {
      pkcSubclass = "string data, right truncation";
      break;
    }
    case data_exception::eSUBSTRING_ERROR:
    {
      pkcSubclass = "substring error";
      break;
    }
    case data_exception::eTRIM_ERROR:
    {
      pkcSubclass = "trim error";
      break;
    }
    case data_exception::eUNTERMINATED_C_STRING:
    {
      pkcSubclass = "unterminated C string";
      break;
    }
    case dynamic_sql_error::eCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED:
    {
      pkcSubclass = "cursor specification cannot be executed";
      break;
    }
    case dynamic_sql_error::eINVALID_DESCRIPTOR_COUNT:
    {
      pkcSubclass = "invalid descriptor count";
      break;
    }
    case dynamic_sql_error::eINVALID_DESCRIPTOR_INDEX:
    {
      pkcSubclass = "invalid descriptor index";
      break;
    }
    case dynamic_sql_error::ePREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION:
    {
      pkcSubclass = "prepared statement not a cursor specification";
      break;
    }
    case dynamic_sql_error::eRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION:
    {
      pkcSubclass = "restricted data type attribute violation";
      break;
    }
    case dynamic_sql_error::eUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC:
    {
      pkcSubclass = "using clause does not match dynamic parameter specifications";
      break;
    }
    case dynamic_sql_error::eUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC:
    {
      pkcSubclass = "using clause does not match target specifications";
      break;
    }
    case dynamic_sql_error::eUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS:
    {
      pkcSubclass = "using clause required for dynamic parameters";
      break;
    }
    case dynamic_sql_error::eUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS:
    {
      pkcSubclass = "using clause required for result fields";
      break;
    }
    case feature_not_supported::eMULTIPLE_ENVIRONMENT_TRANSACTIONS:
    {
      pkcSubclass = "multiple environment transactions";
      break;
    }
    case transaction_rollback::eINTEGRITY_CONSTRAINT_VIOLATION:
    {
      pkcSubclass = "integrity constraint violation";
      break;
    }
    case transaction_rollback::eSERIALIZATION_FAILURE:
    {
      pkcSubclass = "serialization failure";
      break;
    }
    case transaction_rollback::eSTATEMENT_COMPLETION_UNKNOWN:
    {
      pkcSubclass = "statement completion unknown";
      break;
    }
    case warning::eCURSOR_OPERATION_CONFLICT:
    {
      pkcSubclass = "cursor operation conflict";
      break;
    }
    case warning::eDISCONNECT_ERROR:
    {
      pkcSubclass = "disconnect error";
      break;
    }
    case warning::eIMPLICIT_ZERO_BIT_PADDING:
    {
      pkcSubclass = "implicit zero-bit padding";
      break;
    }
    case warning::eINSUFFICIENT_ITEM_DESCRIPTOR_AREAS:
    {
      pkcSubclass = "insufficient item descriptor areas";
      break;
    }
    case warning::eNULL_VALUE_ELIMINATED_IN_SET_FUNCTION:
    {
      pkcSubclass = "null value eliminated in set function";
      break;
    }
    case warning::ePRIVILEGE_NOT_GRANTED:
    {
      pkcSubclass = "privilege not granted";
      break;
    }
    case warning::ePRIVILEGE_NOT_REVOKED:
    {
      pkcSubclass = "privilege not revoked";
      break;
    }
    case warning::eQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA:
    {
      pkcSubclass = "query expression too long for information schema";
      break;
    }
    case warning::eSEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA:
    {
      pkcSubclass = "search condition too long for information schema";
      break;
    }
    case warning::eSTRING_DATA_RIGHT_TRUNCATION:
    {
      pkcSubclass = "string data, right truncation";
      break;
    }
  }
  return pkcSubclass;
  
}  // getSubclass()
